! This example shows how null values are used.
subroutine check( thisNullable )
  use dal
  type(NullableT), intent(in) :: thisNullable

  write(*,*) "Null defined?: ", nullDefined( thisNullable ), nullType( thisNullable )

end subroutine check

program example_nullvalues

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr1, arr2
  type(TableT) tab
  type(ColumnT) col1, col2
  integer(kind=int32), dimension(:), pointer :: i32
  real(kind=double), dimension(:), pointer :: r64
  integer(kind=int32), dimension(:,:,:), pointer :: a1, a2
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr1 = addArray(set, "array1", INTEGER32, dimensions=s )
  arr2 = addArray(set, "array2", arrayDataType( arr1 ), dimensions=s )

  ! fill with unique numbers
  a1 => int32Array3Data(arr1)
  a2 => int32Array3Data(arr1)

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a1(i,j,k) = n
        a2(i,j,k) = a1(i,j,k) + 1
        n = n + 1
      end do
    end do
  end do

  call setNullValue( arr1, 999999 )
  call check( nullable( arr1 ) )

  call setToNull( arr1, 0 )	! Set the first element of array arr1 to null.
      				! Would have given an error, if the null
      				! value of array arr1 had not been set. 

  if( nullType( arr1 ) .eq. INTEGER_NULL ) then ! 
    write(*,*) "Using null value of arr1, in arr2"
    call setNullValue( arr2, intNullValue( arr1 ))
  else
    call setNullValue( arr2, 999999 )
  end if

  call check( nullable( arr2 ) )

  call setToNull( arr2, 1 )	! Set the second element of array arr2 to null.
				! Would have given an error, if the null
				! value of array arr2 had not been set. 

  call release(arr1)
  call release(arr2)

  tab = addTable(set,"some table",100)

  col1 = addColumn(tab,"int32",INTEGER32,units="m",comment="in32 column")

  i32 => int32Data(col1)
  do i=0,numberOfRows(tab)-1
    i32(i) = 3*i
  end do
  call setNullValue( col1, 999999 )
  call check( nullable( col1 ) )

  call setToNull( col1, 0 )	! Set the first element of column col1 to null.


  col2 = addColumn(tab,"real64",REAL64,units="hm",comment="real64 column")
  r64 => real64Data(col2)
  do i=0,numberOfRows(tab)-1
    r64(i) = 0.25*i
  end do

  ! col is a non-integer column and it would be an
  ! an error to call setNullValue().
  call check( nullable( col2 ) )

  call setToNull( col2, 0 )	! Set the first element of column col2 to null.

  if( hasNulls( col2 ) ) then
    do i=0,numberOfRows(tab)-1
      if( isNull( col2, i ) ) then
        write(*,*) "element", i, "is null"
      else
        write(*,*) "element", i, "is", r64(i)
      endif
    end do
  endif

  call release(col1)
  call release(col2)
  call release(set)

end program example_nullvalues
