//  Giuseppe Vacanti (cosine science & computing bv)
//  September 13, 2002
// 
//   $Id: DataSetTracker.cc,v 1.2 2004/03/05 14:43:02 eojero Exp $

#include<DataSetTracker.h>
#include<DataSetRecord.h>
#include<errstr.h>

#include<numeric>
#include<algorithm>
#include<ostream>

DataSetTracker::DataSetTracker() {}

DataSetTracker::~DataSetTracker() {
    for(map<string, DataSetRecord *>::iterator K = db_.begin();
	    K != db_.end(); ++K)
	delete (*K).second;
}


DataSetRecord & DataSetTracker::make_record(std::string const & name){
    if(has_record(name)){
	DataSetRecord & r = get_record(name);
	return r;
    } else {
	DataSetRecord * p = new DataSetRecord(name);
	db_[name] = p;
	DataSetRecord & r = *p;
	return r;
    }
}
	
DataSetRecord & DataSetTracker::get_record(std::string const & name) const{
    if(has_record(name)){
	return get_record(name);
    } else
	errstr << "There is no record for the dataset " << name
	       << Sas::error("NoDataSetRecord");
}

bool DataSetTracker::has_record(std::string const & name) const{
    return db_.find(name) != db_.end();
}

namespace {
    unsigned long count_active(unsigned long n, pair<string, DataSetRecord *> const & p){
	const DataSetRecord * r = p.second;
	if(r->active())
	    ++n;
	return n;
    }

    void print_active(pair<string, DataSetRecord *> const & p){
	const DataSetRecord * r = p.second;
	if(r->active())
	    std::cout << "Active set: " << r->name() << std::endl;
    }
    void print(pair<string, DataSetRecord *> const & p){
	const DataSetRecord * r = p.second;
	std::cout << " Set: " << r->name() << std::endl;
    }
}

unsigned long DataSetTracker::count() const {
    return accumulate(db_.begin(), db_.end(), 0, count_active);
}

void DataSetTracker::printOn(ostream & os) const {
    for(DataSetTracker::const_iterator K = begin();
	K != end(); ++K){
	DataSetRecord & r = (*K).second;
	ostream << "dataset = " << r.name() << " count = " << r.count()
		<< " active = " << r.active() << std::endl;
    }
}

DataSetTracker::iterator begin() {
    return db_.begin();
}

DataSetTracker::const_iterator begin() const {
    return db_.begin();
}

DataSetTracker::iterator end() {
    return db_.end();
}

DataSetTracker::const_iterator end() const {
    return db_.end();
}



