//-----------------------------------------------------------------------------
//
//      XMM Science Analysis System (SAS)
//      (c) 1998 European Space Agency
//
//-----------------------------------------------------------------------------
//
// Slicer.h defines the abstract class Slicer that represents a memory slicer manager.
//
// DESCRIPTION:
//
//
// USAGE EXAMPLES:
//
// CHANGE HISTORY:
//  MT, 1998-10-01: 0.1.0: - first version
//
// AUTHOR:
//  M.Thomas
//
// REFERENCES:
//
// $Header: /sasbuild/repositories/sasdev/sas/packages/dal/src/highlow/Slicer.h,v 1.2 2004/03/05 14:43:02 eojero Exp $
///
#ifndef __SLICER_H
#define __SLICER_H


#include "DalChild.h"
#include "HiLowDataStorage.h"


class Slicer;
class SliceValues;


class Slice : public virtual Child<Slicer>,
       public virtual Child<Slice>,
       public virtual Seekable<unsigned long>
{
public :
  
  Slice();
  virtual ~Slice() {}
  enum SliceType { Super, Sub, Min };
  virtual SliceType type() const = 0;
  virtual int8*    int8Values() = 0;
  virtual uint16*   uint16Values() = 0;
  virtual int16*   int16Values() = 0;
  virtual uint32*   uint32Values() = 0;
  virtual int32*   int32Values() = 0;
  virtual real32*  real32Values() = 0;
  virtual real64*  real64Values() = 0;
  virtual bool8*   boolValues() = 0;
  virtual char*    stringValues() = 0;
  virtual void  copyBytes( Slice * slice ) = 0;
  virtual void   copyBytes( Slice * slice, unsigned long from, unsigned long count ) = 0;
  virtual void  copyBytes( SliceValues * sliceValues ) = 0;
  virtual void   copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count ) = 0;
  virtual void   copyBytes( SliceValues * sliceValues, unsigned long from, unsigned long count, unsigned long to ) = 0;
  virtual void  copyBytes( unsigned long from, unsigned long to, unsigned long count ) = 0;
  virtual void  copyBoolBytes( bool8 * values ) = 0;
  virtual void   copyBoolBytes( bool8 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyInt8Bytes( int8 * values ) = 0;
  virtual void   copyInt8Bytes( int8 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyUint16Bytes( uint16 * values ) = 0;
  virtual void   copyUint16Bytes( uint16 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyInt16Bytes( int16 * values ) = 0;
  virtual void   copyInt16Bytes( int16* values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyUint32Bytes( uint32 * values ) = 0;
  virtual void   copyUint32Bytes( uint32 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyInt32Bytes( int32 * values ) = 0;
  virtual void   copyInt32Bytes( int32 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyReal32Bytes( real32 * values ) = 0;
  virtual void   copyReal32Bytes( real32 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyReal64Bytes( real64 * values ) = 0;
  virtual void   copyReal64Bytes( real64 * values, unsigned long from, unsigned long count ) = 0;
  virtual void  copyStringBytes( char * values ) = 0;
  virtual void   copyStringBytes( char * values, unsigned long from, unsigned long count ) = 0;
  void   incRefs();
  void   decRefs();
  unsigned long  refs() const;
private :
  unsigned long _refs;
};


class Slicer : public DalChild<HiLowDataStorage>
{
public :
  Slicer( HiLowDataStorage * parent );
  ~Slicer();
  void release(
        unsigned long from,
        unsigned long count );
  
  Slice * slice(
  unsigned long from,
  unsigned long count );
  
  void insertBytes(
     unsigned long pos,
     unsigned long count );
  
  void deleteBytes(
     unsigned long from,
     unsigned long count );
  
  void copyBytes(
   unsigned long from,
   unsigned long to,
   unsigned long count );
  
  void releaseAll();
  
  Slice* initialise(
      const Seekable<unsigned long>& oldSeek,
      const Seekable<unsigned long>& seek );
  
private :
  
  DalSeekable<unsigned long> getSliceSize(
          const unsigned long  from,
          unsigned long   count ) const;
  
  // Forbid
  Slicer & operator=( const Slicer & );
  Slicer( const Slicer &);
  
  std::vector<Slice*> _slices;
  
};

#endif



