using namespace std;
#include<cassert>

#include <cstdio>
#include <iostream>


#include <Dal.h>
#include "StopWatch.h"

void
fill( unsigned long nRows )
{

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  Table* tab = set->addTable("some table",nRows);
  Column *col;
  ColumnData *d;

  col = tab->addColumn("int8",Column::Int8,"int8 data","mm");
  assert( !col -> nullDefined() );
  col -> nullValue( long(127) );
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == 127 );
  d = col->data();
  int8 *i8 = d->int8Data();
  for (unsigned long i = 0; i < nRows; ++i)
  {
    i8[i] = i % 100;
  } // for  
  assert( !d -> hasNulls() );
  assert( !d -> isNull( 0 ) );
  d -> setToNull( 0 );
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ) );
  d -> setToNull( nRows - 1 );
  assert( d -> isNull( nRows - 1 ));
  assert( i8[ 0 ] == col -> intNullValue() );
  assert( i8[ nRows - 1 ] == col -> intNullValue() );
  delete d;

  col = tab->addColumn("int16",Column::Int16,"int16 data","cm");
  assert( !col -> nullDefined() );
  col -> nullValue( long(32767) );
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == 32767 );
  d = col->data();
  int16 *i16 = d->int16Data();
  for (unsigned long i = 0; i < nRows; ++i)
    i16[i] = 2*i;
  assert( !d -> hasNulls() );
  assert( !d -> isNull( 0 ) );
  d -> setToNull( 0 );
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ) );
  d -> setToNull( nRows - 1 );
  assert( d -> isNull( nRows - 1 ));
  assert( i16[ 0 ] == col -> intNullValue() );
  assert( i16[ nRows - 1 ] == col -> intNullValue() );
  delete d;

  col = tab->addColumn("int32",Column::Int32,"int32 data","dm");
  assert( !col -> nullDefined() );
  col -> nullValue( long(-99999999) );
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == -99999999 );
  d = col->data();
  int32 *i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    i32[i] = 3*i;
  assert( !d -> hasNulls() );
  assert( !d -> isNull( 0 ) );
  d -> setToNull( 0 );
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ) );
  d -> setToNull( nRows - 1 );
  assert( d -> isNull( nRows - 1 ));
  assert( i32[ 0 ] == col -> intNullValue() );
  assert( i32[ nRows - 1 ] == col -> intNullValue() );
  delete d;

  col = tab->addColumn("real32",Column::Real32,"real32 data","m");
  assert( col -> nullDefined() );
  d = col->data();
  real32 *r32 = d->real32Data();
  for (unsigned long i = 0; i < nRows; ++i)
  {
    r32[i] = 0.5*i;
  } // for
  assert( !d -> hasNulls() );
  d -> setToNull( 0 );
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ) );
  d -> setToNull( nRows - 1 );
  assert( d -> isNull( nRows - 1 ) );
  delete d;

  col = tab->addColumn("real64",Column::Real64,"real64 data","Dm");
  assert( col -> nullDefined() );
  d = col->data();
  real64 *r64 = d->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
  {
    r64[i] = 0.25*i;
  } // for  
  assert( !d -> hasNulls() );
  d -> setToNull( 0 );
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ) );
  d -> setToNull( nRows - 1 );
  assert( d -> isNull( nRows - 1 ) );
  delete d;

  dataSetServer->close(set);

}


template<class T>
void checkValue( ColumnData * coldat, unsigned long i, T value, T original )
{
  if( !coldat -> isNull( i ) )
  {
  	assert( value == original );
  }
  else
  {
  	cout << "Null detected: " << coldat -> parent() -> name() << " " << i << endl;
  } // if
}


void
check( unsigned long nRows )
{

  DataSet* set = dataSetServer->open("test.dat",DataSet::Read);
  Table* tab = set -> table("some table");
  Column *col;
  ColumnData *d;

  col = tab->column("int8",Column::Read);
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == 127 );
  d = col->data();
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ));
  assert( d -> isNull( nRows - 1 ));
  int8 *i8 = d->int8Data();
  assert( i8[ 0 ] == col -> intNullValue() );
  assert( i8[ nRows - 1 ] == col -> intNullValue() );
  for (unsigned long i = 0; i < nRows; ++i)
    checkValue( d, i, i8[i], int8( i % 100 ) );
  delete d;

  col = tab->column("int16",Column::Read);
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == 32767 );
  d = col->data();
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ));
  assert( d -> isNull( nRows - 1 ));
  int16 *i16 = d->int16Data();
  assert( i16[ 0 ] == col -> intNullValue() );
  assert( i16[ nRows - 1 ] == col -> intNullValue() );
  for (unsigned long i = 0; i < nRows; ++i)
    checkValue( d, i, i16[i], int16( 2*i ));
  delete d;

  col = tab->column("int32",Column::Read);
  assert( col -> nullDefined() );
  assert( col -> intNullValue() == -99999999 );
  d = col->data();
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ));
  assert( d -> isNull( nRows - 1 ));
  int32 *i32 = d->int32Data();
  assert( i32[ 0 ] == col -> intNullValue() );
  assert( i32[ nRows - 1 ] == col -> intNullValue() );
  for (unsigned long i = 0; i < nRows; ++i)
    checkValue( d, i, i32[i], int32( 3*i ));
  delete d;

  col = tab->column("real32",Column::Read );
  assert( col -> nullDefined() );
  d = col->data();
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ));
  assert( d -> isNull( nRows - 1 ));
  real32 *r32 = d->real32Data();
  for (unsigned long i = 0; i < nRows; ++i)
  {
    checkValue( d, i, r32[i], real32( 0.5*i ));
  } // if
  delete d;

  col = tab->column("real64",Column::Read );
  assert( col -> nullDefined() );
  d = col->data();
  assert( d -> hasNulls() );
  assert( d -> isNull( 0 ));
  assert( d -> isNull( nRows - 1 ));
  real64 *r64 = d->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
    checkValue( d, i, r64[i], real64( 0.25*i ));
  delete d;

  dataSetServer->close(set);

}


int main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );


	fill(10);
	check(10);
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	return 0;
}

